;
; cuter.asm
; Who's Cuter? for NES: main
; Copyright 2000 Damian Yerrick
; compile with x816 assembler available from
; http://www.zophar.net/
;
; If you are planning on using this code in a Real NES, put it in an
; UNROM cartridge with horizontal mirroring (Contra not Ikari).
; The iNES board number for UNROM is 2.
;
;
;
; 0000 direct page
; 0080 NT2 replay code direct page area
; 00C0 direct page
; 0100 stack
; 0200 sprite DMA area
; 0300 NT2 replay code main ram area
; 0400 the map
; 0480 second field, used for redraw manager
; 0500 unused
; 0800 ---
; 8000 replay code
; 9000
; e000 nametables and other data tables
; f000
; fffa interrupt vectors
;

                .opt on
                .mem 8
                .index 8
                .org $c000
                .list

;PPU regs
spraddr = $2003
sprdma = $4014
ppuaddr = $2006
ppudata = $2007

face1 = $3e
face2 = $3f
h = $3d
i = $3c
j = $3b
t = face1
hindex = $3a
sortstage = $39

rankings = $7c0


sprbufptr = $e7
repeats1 = $e8
repeats2 = $f0
randseed = $f8
retraces = $fa          ; (16 bits) the number of retraces
control1 = $fc          ; data read from controllers
control2 = $fd
control3 = $fe
control4 = $ff


sprbuf = $200           ; sprite dma communication area


;
; main()
;
; Nintendo wrote the first part of the init code in _Duck_Hunt_.
; Kevin Horton and Chris Covell played with it before I put it in.
;

                .db 13,10,13,10
                .incbin "about.txt"

resetcode       cld
                sei
                jsr wait4vbl

                lda #$20
                sta ppuaddr
                ldx #1          ;initialize sound chip
                stx $4015
                dex             ;0 -> A
                txa             ;(A will contain 0 for most of init)
                sta ppuaddr
                dex             ;255 -> S
                stx $4003
                txs

                sta $2000       ;turn off screen display
                sta $2001
                sta $2005
                sta $2005
                sta $4015       ;turn off sound


;clear the NES's CPURAM
                tay
-
                sta 0,y
                sta $100,y
                sta $300,y
                sta $400,y
                sta $500,y
                sta $600,y
                sta $700,y
                dey
                bne -

;clear the second nametable
                lda #$24
                sta ppuaddr
                lda #0
                sta ppuaddr
                lda #1
                ldy #0
                jsr clearnametable
                lda #0

;clear out sprites
                sta spraddr
                tax
-               lda #$ef
		sta sprbuf,x   ;Y = 241
		inx
		lda #$b6
                sta sprbuf,x   ;index = blank
		inx
                lda #0
		sta sprbuf,x   ;no flip, front, color 0
		inx
                sta sprbuf,x   ;X = 0
		inx
                bne -

                lda $2002       ;load ascii characters
                lda #0          ;and logo
                sta ppuaddr
                sta ppuaddr
                lda #32
                jsr imgcopy
                lda #34
                jsr imgcopy

                lda #$20        ;draw title screen
                sta ppuaddr
                lda #0
                sta ppuaddr
                lda #48
                jsr imgcopy

                lda #$27        ;draw second nametable
                sta ppuaddr
                lda #$f0
                sta ppuaddr
                ldx #8
                lda #$ff
-
                sta ppudata
                dex
                bne -

                ldx #0
                stx 0
                ldx #titlepal-pals
                jsr putpal
                jsr wait4endvbl
                jsr wait4vbl

                lda #183        ;set up sprite 0
                sta sprbuf
                lda #95
                sta sprbuf+1
                lda #0
                sta sprbuf+2
                sta sprbuf+3
                sta ppuaddr
                sta ppuaddr
                lda #%10000000
                sta $2000
                lda #%00011110
                sta $2001
                lda #>sprbuf
                sta sprdma
                lda #0
                sta 6           ;character drawing pos
                lda #$27
                sta 7           ;character drawing nt
                lda #<textual
                sta 8           ;textaddr lo
                lda #>textual
                sta 9           ;textaddr hi
                lda #4
                sta 10          ;spaces left
                lda #$81
                sta 5           ;$2000 value
                lda #$fc
                sta 4           ;xscroll
                
--
                lda #0
                sta ppuaddr
                sta ppuaddr
                lda #0
                sta $2005
                sta $2005
-
                bit $2002       ;wait for spr0 nohit
                bvs -
-
                bit $2002       ;wait for spr0 hit
                bvc -

                lda 5
                sta $2000
                lda 4
                sta $2005
                lda #0
                sta $2005

                bit $2002
                bvc -

                ldy #217
                ldx #3
-
                dey
                bne -
                dex
                bne -
                lda #0
                sta $2000
                sta $2005
                sta $2005
                         
                jsr wait4vbl
                lda #2
                clc
                adc 4
                sta 11
                bcc +
                lda 5
                eor #1
                sta 5
                lda 11
+
                eor 4
                and #$f0
                beq +
                jsr nextchar
+
                lda 11
                sta 4
                jsr ReadJPad
                lda control1
                and #$80
                beq --

                ldx #31         ;load default rankings (mine :-)
-
                lda defaultrank,x
                sta rankings,x
                dex
                bpl -
--
                lda #<msg1
                sta 4
                lda #>msg1
                sta 5
                lda #255
                sta 6
                jsr dointer
                
                lda #0
                sta sortstage
                jsr shellsort

                lda #<msg2
                sta 4
                lda #>msg2
                sta 5
                lda #0
                sta 6
                jsr dointer

                lda #<msg3
                sta 4
                lda #>msg3
                sta 5
                lda #16
                sta 6
                jsr dointer

                lda #0
                sta i
-
                ldx i
                lda rankings,x
                sta face1
                inx
                lda rankings,x
                sta face2
                inx
                stx i
                jsr disp
                lda i
                cmp #32
                bcc -
                
                jmp --

                ;    01234567890123456789012345678901
msg1
                .db "  Press the + Control Pad in    "
                .db "  the direction of the cuter    "
                .db "  picture.",0
msg2
                .db "  and the envelope please...    "
                .db "  The ugly ones:",0
msg3
                .db "  The cute ones:",0

               






nextchar
                lda 10
                beq +
                dec 10
                lda #32
                bne ++
+
                ldy #0
                lda (8),y
                inc 8
                bne ++
                inc 9
++
                cmp #10
                bne +
                lda #16
                sta 10
                lda #32
+
                sta 2
                asl a           ;Compute the address of the glyph.
                asl a
                sta 0
                lda #>fonttable
                adc #0
                sta 1

                ldy #3          ;Copy the glyph to the nametable.
-
                lda 7           ;Find the tile line.
                sta ppuaddr
                tya
                asl a
                asl a
                asl a
                asl a
                asl a
                ora 6
                sta ppuaddr

                lda (0),y       ;Draw the left half.
                lsr a
                lsr a
                lsr a
                lsr a
                ora #$10
                sta ppudata
                lda (0),y       ;Draw the right half.
                and #$0f
                ora #$10
                sta ppudata

+
                dey
                bpl -

                lda 6           ;Advance one character position.
                clc
                adc #2
                and #$1e
                sta 6
                bne +
                lda #4
                eor 7
                sta 7
+
                rts

.comment
................
...1111111111...
...1111111111...
...1111111111...
...1111111111...
...1111111111...
...1111111111...
................
...2222222222...
...2222222222...
...2222222222...
...2222222222...
...2222222222...
...2222222222...
................
.end

fillcard
                lda #12
                sta 0
                ldy #16
--
                ldx #20
-
                sty ppudata
                iny
                dex
                bne -
                ldx #12
                lda #3
-
                sta ppudata
                dex
                bne -
                dec 0
                bne --
                rts

ingameattr
                .db $00,$40,$50,$50,$50,$50,$10,$00
                .db $00,$44,$55,$55,$55,$55,$11,$00
                .db $00,$44,$55,$55,$55,$55,$11,$00
                .db $00,$04,$05,$05,$05,$05,$01,$00
                .db $00,$88,$aa,$aa,$aa,$aa,$22,$00
                .db $00,$88,$aa,$aa,$aa,$aa,$22,$00
                .db $00,$88,$aa,$aa,$aa,$aa,$22,$00
                .db $00,$00,$00,$00,$00,$00,$00,$00


disp
                lda #0
                tax
                sta $2001
                lda $2002
                lda #$3f        ;set palette to white
                sta ppuaddr
                lda #0
                sta ppuaddr
                lda #$30
                sta ppudata


                lda #$20        ;clear out nametables
                sta ppuaddr
                lda #0
                sta ppuaddr
                tay
                lda #3
                ldy #0
                jsr clearnametable
                jsr clearnametable

                lda #$20        ;write the tiles
                sta ppuaddr
                lda #$46
                sta ppuaddr
                jsr fillcard
                lda #$22
                sta ppuaddr
                lda #$06
                sta ppuaddr
                jsr fillcard
                lda #$23
                sta ppuaddr
                lda #$c0
                sta ppuaddr
                ldx #0          ;write the attributes
-
                lda ingameattr,x
                sta ppudata
                inx
                cpx #64
                bcc -

                ldx #111        ;init sprite 0 bankswitch
                stx sprbuf
                ldx #3
                stx sprbuf+1
                ldx #0
                stx sprbuf+2
                ldx sortstage
                inc sortstage
                stx sprbuf+3    ;sprite 0 gives progress counter :-)
                lda #>sprbuf
                sta sprdma

                lda #0          ;copy images
                sta ppuaddr
                sta ppuaddr
                lda #33         ;top
                jsr imgcopy
                lda face1
                jsr imgcopy
                lda #33         ;top
                jsr imgcopy
                lda face2
                jsr imgcopy

-
                jsr ReadJPad
                lda control1
                bne -

                jsr wait4vbl

                lda #$3f
                sta ppuaddr
                lda #0
                sta ppuaddr
                lda #$0f
                sta ppudata
                lda #0
                sta ppudata
                lda #$10
                sta ppudata
                lda #$30
                sta ppudata
                sta ppudata

                ldx face1
                lda imgpals1,x
                sta ppudata
                lda imgpals2,x
                sta ppudata
                lda imgpals3,x
                sta ppudata
                sta ppudata
                ldx face2
                lda imgpals1,x
                sta ppudata
                lda imgpals2,x
                sta ppudata
                lda imgpals3,x
                sta ppudata

                lda #0
                sta ppuaddr
                sta ppuaddr
                sta $2005
                sta $2005
                sta $2000
                lda #%00011110
                sta $2001

disploop
                lda #0
                sta $2000       ;first pattable
-
                bit $2002
                bvs -
-
                bit $2002
                bvc -

                lda #$10        ;second pattable
                sta $2000
                jsr wait4vbl

                jsr ReadJPad
                lda control1
                and #$08        ;up
                beq +
                clc
                rts
+
                lda control1
                and #$04        ;down
                beq disploop
                sec
                rts

                .incsrc "shelsort.asm"

shellcompare
;                lda face2
;                cmp face1
;                rts
                jmp disp


div10
                .db "          1111111111222222222233333333334444444444"
                .db "55555555556666666666777777777788888888889999999999"
mod10
                .db "01234567890123456789012345678901234567890123456789"
                .db "01234567890123456789012345678901234567890123456789"

defaultrank
                .db  9,10,14, 6,13,19, 2, 1,26, 0, 3,16,17,20,18,30
                .db 22,11,21,28, 4, 7, 5,25,24, 8,12,31,15,29,23,27

;
; dointer
; Do between-phase screens.
; 4 pointer to 0-term string to draw at ppu $2120
; 6 ranking base (>= 32: no ranking)
;
dointer
                lda #0
                ldy #$20
                sta $2001
                sty ppuaddr
                sta ppuaddr
                lda #49         ;inter nametable
                jsr imgcopy
                lda #$21
                sta ppuaddr
                lda #$20
                sta ppuaddr
                ldy 4
                lda 5
                jsr puts
                ldx #interpal-pals
                jsr putpal
                lda #0
                sta ppuaddr
                sta ppuaddr
                sta $2000
                lda #32         ;load ascii table
                jsr imgcopy
                lda #35         ;load small logo
                jsr imgcopy
                lda #0
                sta 0
                ldx #interpal-pals
                jsr putpal

                lda 6           ;draw ranking?
                cmp #32
                bcs +
                lda #16
                sta 3
--
                lda #0          ;find the tileline to draw name on
                sta 1
                lda 3
                clc
                adc #11
                lsr a
                ror 1
                lsr a
                ror 1
                lsr a
                ror 1
                ora #$20
                sta ppuaddr
                lda 1
                sta ppuaddr
                lda #32
                sta ppudata
                sta ppudata
                lda #32         ;write the number ranking
                sec
                sbc 6
                tax
                lda div10,x
                sta ppudata
                lda mod10,x
                sta ppudata
                lda #'.'
                sta ppudata
                lda #32
                sta ppudata
                ldx 6
                lda #0
                tay
                sta 1
                lda rankings,x
                asl
                rol 1
                asl
                rol 1
                asl
                rol 1
                asl
                rol 1
                adc #<imgnames
                sta 0
                lda 1
                adc #>imgnames
                sta 1
-
                lda (0),y
                sta ppudata
                iny
                cpy #16
                bcc -

                inc 6
                dec 3
                bne --
+
                jsr wait4vbl
                lda #0
                sta ppuaddr
                sta ppuaddr
                lda #%00001010  ;bg, but no sprites!
                sta $2001
-
                jsr ReadJPad
                lda control1
                bne -
-
                jsr ReadJPad
                lda control1
                beq -
                rts

FAQ2      EQU $6A
TEMP      EQU 0

;
; ltoa
; Convert a long stored in FAQ2..FAQ2+3 to a 10-digit decimal in
; $101-$10a.  (I know that's stack memory, but if you're not using
; recursion, it should be ok.)
;
ltoa
          ldx #10               ;number base
          stx faq2+4
          ldx #10               ;max # digits to output
-
          jsr div32
          ora #48
          sta $0100,x     ;Temporary area at the top of stack
          dex
          bne -     
          rts


;
; div32
; Divide long in faq2 by byte in faq2+4.
; Result:  faq2 = quotient; A = remainder; Y = 0; X untouched
;
div32     LDA #0
          LDY #32
-
          ASL FAQ2
          ROL FAQ2+1
          ROL FAQ2+2
          ROL FAQ2+3
          ROL
          CMP FAQ2+4
          BCC +
          SBC FAQ2+4
          INC FAQ2
+
          DEY
          BNE -
          RTS























;
; ===============================
;   END OF MAIN BODY OF PROGRAM
; ===============================
;

;
; PlayWav
; Plays a sample.
;


;
; imgcopy
; 
; Unpack an data block to the current PPU address.
; Call it with the screen off.
; in
;   A = file number
; out
;   A = number associated with file number
;   X = file number * 4
; Trashes all regs and memory 0-3.
; Leaves ppuaddr at the current data block.
;
imgcopy
                asl
                asl
                pha
                tax
                lda imgtable,x
                sta 0
                lda imgtable+1,x
                sta 1
                lda imgtable+2,x
                jsr trainer
                jsr PKB_unpackblk
                pla
                tax
                lda imgtable+3,x
                rts

;
; MakeRepeats
; Translates controller A into autorepeats.
; Trashes: A X Y

MakeRepeats     lda control1
                ldx #7
-
                lsr a
                bcs +
                ldy #0
                beq ++
+
                ldy repeats1,x
                iny
                cpy #18
                bcc ++
                ldy #16
++
                sty repeats1,x
                dex
                bpl -

                lda control2
                ldx #7
-
                lsr a
                bcs +
                ldy #0
                beq ++
+
                ldy repeats2,x
                iny
                cpy #18
                bcc ++
                ldy #16
++
                sty repeats2,x
                dex
                bpl -

                rts



;
; rand
; Generates a pseudorandom number from tons of variables;
; returns it in a
; Trashes y
;

rand
                lda randseed
                inc randseed
                eor randseed+1
                tay
                lda randdata,y
                rts


;
; wait4vbl
; waits for vertical blank
;

wait4vbl        bit $2002
                bpl wait4vbl
                rts


;
; wait4endvbl
; waits for end of vertical blank
;

wait4endvbl     bit $2002
                bmi wait4endvbl
                rts


;
; puts
; There is a C-style string in (AAYY). Print it to wherever ppuaddr points.
; Trashes: 0 1
; output: A = 0, Y = string length
;

puts
                sty 0
                sta 1
                ldy #0
-               lda (0),y
                beq +
                sta ppudata
                iny
                bne -
+               rts


;
; puthex
; Writes the number in A to ppudata.
;

puthex          pha
                pha
                lsr a
                lsr a
                lsr a
                lsr a
                tax
                lda hexdigits,x
                sta ppudata
                pla
                and #$0f
                tax
                lda hexdigits,x
                sta ppudata
                pla
                rts
hexdigits       .db "0123456789ABCDEF"




;
; clearnametable
; Set the nametable at ppuaddr to A and its attributes to Y.
;

clearnametable
                ldx #240
-
                sta ppudata
                sta ppudata
                sta ppudata
                sta ppudata
                dex
                bne -
                ldx #16
-
                sty ppudata
                sty ppudata
                sty ppudata
                sty ppudata
                dex
                bne -
                rts


;
; PKB_unpackblk
;

                .incsrc "unpkb.asm" ;packbits decompression

;
; putpal
; Copies 32 bytes from (pals,x) to the palette.  Subtracts $000
; for fading.  Call during VBL.
;

putpal
                lda #$3F
                sta ppuaddr
                lda #0
                sta ppuaddr
                
                ldy #$20
-               lda pals,x
                sec
                sbc 0
                bpl +
                lda #0
+
                sta ppudata
                inx
                dey
                bne -
                rts


;
; ReadJPad
; Poll the controllers.
;

ReadJPad
; Read controllers 1 and 3.
                lda #1
                sta $4016
                lda #0
                sta $4016
                ldx #8
-               lda $4016
                lsr a
                rol control1
                dex
                bne -
                ldx #8
-               lda $4016
                lsr a
                rol control3
                dex
                bne -

; Read controllers 2 and 4.
                lda #1
                sta $4016
                lda #0
                sta $4016
                ldx #8
-               lda $4017
                lsr a
                rol control2
                dex
                bne -
                ldx #8
-               lda $4017
                lsr a
                rol control4
                dex
                bne -
                rts

;
; DATA TABLES
;

                .pad $d000
fonttable
                .incbin "scrfont.bin"
randdata
                .incbin "randnos.bin"

;
; graphics table
; lda #foo sta ppuaddr lda #bar sta ppuaddr lda imgbank jsr imgcopy
;
                .incsrc "grtable.asm"

; palette for title screen and game
pals

titlepal        .db $0f,$30,$10,$00,$0f,$06,$16,$26,$0f,$0a,$1a,$2a,$0f,$0a,$1a,$10
                .db $0f,$27,$16,$08,$0f,$27,$02,$08,$0f,$27,$16,$08,$0f,$27,$0a,$08
interpal        .db $0f,$1d,$00,$10,$0f,$0f,$0f,$0f,$0f,$0f,$0f,$0f,$0f,$0f,$0f,$0f
                .db $0f,$1d,$00,$10,$0f,$0f,$0f,$0f,$0f,$0f,$0f,$0f,$0f,$0f,$0f,$0f
ingamepal       .db $0f,$00,$10,$30,$0f,$06,$16,$26,$0f,$0a,$1a,$2a,$0f,$02,$12,$22
                .db $0f,$27,$16,$08,$0f,$27,$02,$08,$0f,$27,$16,$08,$0f,$27,$0a,$08

textual
                .incbin "demotext.txt"












;The ROM's trainer
                .pad $ff00


;
; trainer
; Load a 16-KB bank into $8000-$BFFF.
;

trainer
                jmp unromtrainer

;
; nmipoint
; Increment retrace.
;

nmipoint
                pha             ;Save all regs.
                txa
                pha
                tya
                pha


;Handle the retrace counter.
                inc retraces
                bne +
                inc retraces+1
+

                pla             ;Restore all regs and exit.
                tay
                pla
                tax
                pla
irqpoint
                rti

resetpoint
                lda #1
                sta $a000
                jmp resetcode

unromtrainer
                sta $fff0
                rts

mmc1trainer
                jmp mmc1trainer ;not implemented yet

mmc3trainer                     ;untested
                pha
                and #$7f
                asl
                pha
                lda #0
                sta $8000
                pla
                pha
                asl
                sta $8001
                lda #1
                sta $8000
                pla
                ora #1
                sta $8001
                pla
                rts



;interrupt table
                .pad $fff0

                ; Two values placed on a data bus are anded together.
                ; $ff values prevent bus conflicts on mapper writes.
                ; see also: undoc instruction "sax"
                .dw $ffff,$ffff,$ffff,$ffff,$ffff
                .dw nmipoint
                .dw resetpoint
                .dw irqpoint
